///////////////////////////////////////////////////////////////////////////////
//
// Служебный модуль с реализацией сценариев обработки файлов <СинхронизацияОбъектовМетаданныхИФайлов>
//
///////////////////////////////////////////////////////////////////////////////

Перем КорневойПуть;

// ИмяСценария
//	Возвращает имя сценария обработки файлов
//
// Возвращаемое значение:
//   Строка   - Имя текущего сценария обработки файлов
//
Функция ИмяСценария() Экспорт
	
	Возврат "СинхронизацияОбъектовМетаданныхИФайлов";
	
КонецФункции // ИмяСценария()

// ОбработатьФайл
//	Выполняет обработку файла
//
// Параметры:
//  АнализируемыйФайл		- Файл - Файл из журнала git для анализа
//  КаталогИсходныхФайлов  	- Строка - Каталог расположения исходных файлов относительно каталог репозитория
//  ДополнительныеПараметры - Структура - Набор дополнительных параметров, которые можно использовать
//  	* Лог  					- Объект - Текущий лог
//  	* ИзмененныеКаталоги	- Массив - Каталоги, которые необходимо добавить в индекс
//		* КаталогРепозитория	- Строка - Адрес каталога репозитория
//		* ФайлыДляПостОбработки	- Массив - Файлы, изменившиеся / образовавшиеся в результате работы сценария
//											и которые необходимо дообработать
//
// Возвращаемое значение:
//   Булево   - Признак выполненной обработки файла
//
Функция ОбработатьФайл(АнализируемыйФайл, КаталогИсходныхФайлов, ДополнительныеПараметры) Экспорт
	
	Лог = ДополнительныеПараметры.Лог;
	НастройкиСценария = ДополнительныеПараметры.Настройки.Получить(ИмяСценария());
	ЭтоФайлОписанияКонфигурации = ТипыФайлов.ЭтоФайлОписанияКонфигурации(АнализируемыйФайл) 
									ИЛИ ТипыФайлов.ЭтоФайлОписанияКонфигурацииEDT(АнализируемыйФайл);

	Если АнализируемыйФайл.Существует() И ЭтоФайлОписанияКонфигурации Тогда
		
		Лог.Информация("Обработка файла '%1' по сценарию '%2'", АнализируемыйФайл.ПолноеИмя, ИмяСценария());
		ЭтоEDT = ТипыФайлов.ЭтоФайлОписанияКонфигурацииEDT(АнализируемыйФайл);

		Если СинхронизироватьМетаданныеИФайлы(АнализируемыйФайл.ПолноеИмя, ДополнительныеПараметры.ИзмененныеКаталоги, ЭтоEDT) Тогда
			
			ДополнительныеПараметры.ИзмененныеКаталоги.Добавить(АнализируемыйФайл.ПолноеИмя);
			
		КонецЕсли;
		
		Возврат Истина;
		
	КонецЕсли;
	
	Возврат Ложь;
	
КонецФункции // ОбработатьФайл()

Функция СинхронизироватьМетаданныеИФайлы(Знач ИмяФайла, УдаленныеФайлы, ЭтоEDT)
	
	СодержимоеФайла = ФайловыеОперации.ПрочитатьТекстФайла(ИмяФайла);
	ДочерниеЭлементы = РегулярныеВыражения.ПолучитьДочерниеЭлементыОписанияКонфигурации(СодержимоеФайла, ЭтоEDT);
	
	Если ДочерниеЭлементы.Количество = 0 Тогда
		Возврат Ложь;
	КонецЕсли;
	
	// Это те объекты, которые есть в описании конфигурации
	ОбъектыМетаданных = ПолучитьОбъектыМетаданных(ДочерниеЭлементы.Совпадения);
	
	// Список базовых типов метаданных
	СписокКаталогов = МетаданныеКонфигурации.КаталогиТиповМетаданных(ЭтоEDT);
	
	
	КорневойПуть = ?(ЭтоEDT, ОбъединитьПути(Новый Файл(ИмяФайла).Путь, ".."), Новый Файл(ИмяФайла).Путь);
	СписокДляУдаления = Новый Массив;
	ЕдиныйТекстОшибки = "";
	ТекстОшибок = Новый Массив;
	
	Для Каждого КаталогСписка Из СписокКаталогов Цикл
		
		Если ПроверитьКаталог(КаталогСписка, ОбъектыМетаданных, СписокДляУдаления, ТекстОшибок) Тогда
			Продолжить;
		КонецЕсли;
		
		Каталог = Новый Файл(ОбъединитьПути(КорневойПуть, КаталогСписка.Значение));
		
		ВсеФайлыОбъектовТипа = Новый Массив;
		// Проверка содержимого каталога
		Если ЭтоEDT Тогда
		 Каталоги = ФайловыеОперации.НайтиКаталоги(Каталог.ПолноеИмя);
		

		 Для каждого Каталог Из Каталоги Цикл
			Файлы = НайтиФайлы(Каталог.ПолноеИмя,"*.mdo", Ложь);
			Для каждого Файл из Файлы Цикл
				ВсеФайлыОбъектовТипа.Добавить(Файл);
			КонецЦикла;
		КонецЦикла;

		Иначе
			ВсеФайлыОбъектовТипа = НайтиФайлы(Каталог.ПолноеИмя, "*.xml");
		КонецЕсли;

		НадоПропустить = Новый ТаблицаЗначений;
		НадоПропустить.Колонки.Добавить("Имя");
		НадоПропустить.Колонки.Добавить("НормализованноеИмя");
		
		Для Каждого ФайлОбъектаТипа Из ВсеФайлыОбъектовТипа Цикл
			
			Если НадоПропустить.Найти(ФайлОбъектаТипа.ИмяБезРасширения) <> Неопределено Тогда
				Продолжить;
			КонецЕсли;
			
			Строка = НадоПропустить.Добавить();
			Строка.Имя = ФайлОбъектаТипа.ИмяБезРасширения;
			Строка.НормализованноеИмя = Нормализовать(ФайлОбъектаТипа.ИмяБезРасширения);
			СтруктураОтбора = Новый Структура("ТипМетаданных, ИмяМетаданных",
					КаталогСписка.Представление, ФайлОбъектаТипа.ИмяБезРасширения);
			СуществующиеОбъекты = ОбъектыМетаданных.НайтиСтроки(СтруктураОтбора);
			Если СуществующиеОбъекты.Количество() Тогда
				
				Для Каждого УдаляемаяСтрока Из СуществующиеОбъекты Цикл
					
					ОбъектыМетаданных.Удалить(УдаляемаяСтрока);
					
				КонецЦикла;
				
				Продолжить;
				
			КонецЕсли;
			
			// остатки файлов надо удалить
			СписокДляУдаления.Добавить(ФайлОбъектаТипа.ПолноеИмя);
			ФайлОбъектаТипаКаталог = Новый Файл(ОбъединитьПути(Каталог.ПолноеИмя, ФайлОбъектаТипа.ИмяБезРасширения));
			СписокДляУдаления.Добавить(ФайлОбъектаТипаКаталог.ПолноеИмя);
			
		КонецЦикла;
		
		ВсеФайлыОбъектовТипа = ФайловыеОперации.НайтиКаталоги(?(ЭтоEDT, Каталог.Путь, Каталог.ПолноеИмя));
		
		Для Каждого ФайлОбъектаТипа Из ВсеФайлыОбъектовТипа Цикл
			
			Если НадоПропустить.Найти(ФайлОбъектаТипа.ИмяБезРасширения) <> Неопределено Тогда
				Продолжить;
			КонецЕсли;
			
			// Ситуация редкая, проверим в самом конце на регистр
			Строки = НадоПропустить.НайтиСтроки(Новый Структура("НормализованноеИмя", Нормализовать(ФайлОбъектаТипа.ИмяБезРасширения)));
			Если Строки.Количество() Тогда
				ОшибкаСтр = СтрШаблон("Имя каталога %1 отличается от метаданного %2. Надо переименовать папку %3",
						ФайлОбъектаТипа.ИмяБезРасширения, Строки[0].Имя, ФайлОбъектаТипа.ПолноеИмя);
				ТекстОшибок.Добавить(ОшибкаСтр);
				Продолжить;
			КонецЕсли;
			
			// Остатки файлов надо удалить
			СписокДляУдаления.Добавить(ФайлОбъектаТипа.ПолноеИмя);
			
		КонецЦикла;
		
	КонецЦикла;
	
	ПеренестиМетаданныеВОшибку(ОбъектыМетаданных, ТекстОшибок);
	
	// Разбор файлов для удаления в строку с ошибкой
	Если СписокДляУдаления.Количество() Тогда
		
		СтрокаОшибки = "Необходимо удалить файлы ";
		СтрокаОшибки = СтрокаОшибки + СтрСоединить(СписокДляУдаления, Символы.ПС + СтрокаОшибки);
		ТекстОшибок.Добавить(СтрокаОшибки);
		
	КонецЕсли;
	
	ЕдиныйТекстОшибки = СтрСоединить(ТекстОшибок, Символы.ПС);
	
	Если НЕ ПустаяСтрока(ЕдиныйТекстОшибки) Тогда
		
		ВызватьИсключение ЕдиныйТекстОшибки;
		
	КонецЕсли;
	
	Возврат Истина;
	
КонецФункции

Функция ПолучитьМассивИсключенийМетаданных()
	
	МассивИсключений = Новый Массив();
	МассивИсключений.Добавить("Sequence");
	
	Возврат МассивИсключений;
	
КонецФункции

// В данном случае для нормализации используются символы с которых не может начинаться имя метаданного в 1С для того,
// чтобы не вызвать пересечение при приведении к регистру
Функция Нормализовать(ИмяМетаданного)
	Возврат ВРег("%" + ИмяМетаданного);
КонецФункции

Функция ПолучитьОбъектыМетаданных(КоллекцияСовпадений)
	
	ОбъектыМетаданных = Новый ТаблицаЗначений;
	ОбъектыМетаданных.Колонки.Добавить("ТипМетаданных");
	ОбъектыМетаданных.Колонки.Добавить("ИмяМетаданных");
	
	Для Каждого ОбъектМетаданных Из КоллекцияСовпадений Цикл
		
		НоваяЗапись = ОбъектыМетаданных.Добавить();
		НоваяЗапись.ТипМетаданных = ОбъектМетаданных.Группы[1].Значение;
		НоваяЗапись.ИмяМетаданных = ОбъектМетаданных.Группы[2].Значение;
		
	КонецЦикла;
	
	ОбъектыМетаданных.Свернуть("ТипМетаданных, ИмяМетаданных", "");
	
	Возврат ОбъектыМетаданных;
	
КонецФункции

Функция ПроверитьКаталог(КаталогСписка, ОбъектыМетаданных, СписокДляУдаления, ТекстОшибок)
	
	ОстановитьПроверкуКаталога = Ложь;
	
	ОбъектыМетаданныхТипа = ОбъектыМетаданных.НайтиСтроки(Новый Структура("ТипМетаданных", КаталогСписка.Представление));
	
	Каталог = Новый Файл(ОбъединитьПути(КорневойПуть, КаталогСписка.Значение));
	
	Если НЕ Каталог.Существует() Тогда
		
		Если ОбъектыМетаданныхТипа.Количество() Тогда
			
			// каталога нет, а должен быть
			ТекстОшибок.Добавить(СтрШаблон("Отсутствует каталог %1", КаталогСписка.Значение));
			
		КонецЕсли;
		
		ОстановитьПроверкуКаталога = Истина;
		
	КонецЕсли;
	
	// Если типа метаданных нет, то просто удаляем каталог
	Если НЕ ОстановитьПроверкуКаталога И НЕ ОбъектыМетаданныхТипа.Количество() Тогда
		
		СписокДляУдаления.Добавить(Каталог.ПолноеИмя);
		ОстановитьПроверкуКаталога = Истина;
		
	КонецЕсли;
	
	Возврат ОстановитьПроверкуКаталога;
	
КонецФункции

Процедура ПеренестиМетаданныеВОшибку(ОбъектыМетаданных, ТекстОшибок)
	
	МассивИсключенийМетаданных = ПолучитьМассивИсключенийМетаданных();
	
	// проверка наличия оставшихся объектов, для которых нет каталогов
	Для Каждого ОбъектМетаданных Из ОбъектыМетаданных Цикл
		
		Если МассивИсключенийМетаданных.Найти(ОбъектМетаданных.ТипМетаданных) <> Неопределено Тогда
			
			Продолжить;
			
		КонецЕсли;
		
		ТекстОшибок.Добавить(СтрШаблон("Отсутствуют файлы для %1.%2", ОбъектМетаданных.ТипМетаданных, ОбъектМетаданных.ИмяМетаданных));
		
	КонецЦикла;
	
КонецПроцедуры
